@@ -73,6 +73,36 @@ class CacheTaggingTest(TestCase):
         cache.invalidate_tags('non_existen_tag')
         self.assertEqual(cache.get('name1', None), None)
 
+    def test_ancestors(self):
+        val1 = cache.get('name1')
+        self.assertEqual(val1, None)
+        if val1 is None:
+            val2 = cache.get('name2')
+            self.assertEqual(val2, None)
+            if val2 is None:
+                val2 = 'val2'
+                cache.set('name2', val2, ('tag2', ), 120)
+            val1 = 'val1' + val2
+            cache.set('name1', val1, ('tag1', ), 120)
+        cache.invalidate_tags('tag2')
+        self.assertEqual(cache.get('name1'), None)
+        self.assertEqual(cache.get('name2'), None)
+
+        cache.set('name2', 'val2', ('tag2', ), 120)
+        val1 = cache.get('name1')
+        self.assertEqual(val1, None)
+        if val1 is None:
+            val2 = cache.get('name2')
+            self.assertEqual(val2, 'val2')
+            if val2 is None:  # val2 is not None, it's only for demonstration
+                val2 = 'val2'
+                cache.set('name2', val2, ('tag2', ), 120)
+            val1 = 'val1' + val2
+            cache.set('name1', val1, ('tag1', ), 120)
+        cache.invalidate_tags('tag2')
+        self.assertEqual(cache.get('name1'), None)
+        self.assertEqual(cache.get('name2'), None)
+
     def test_decorator_cache_page(self):
         resp1 = self.client.get(reverse("cache_tagging_test_decorator"))
         # The first call is blank.
@@ -282,7 +312,7 @@ class CacheTaggingTest(TestCase):
         self.assertEqual(cache.get('name1'), 'value1')
 
         cache.transaction_finish()  # 2
-        self.assertEqual(cache.get('name2', None), None)
+        self.assertEqual(cache.get('name2', None, abort=True), None)
         self.assertEqual(cache.get('name1'), 'value1')
 
         cache.set('name2', 'value2', ('tag2', ), 120)
